"""eXistDACacheManager.py : eXistDA CacheManager product module."""

# -*- coding: ISO-8859-1 -*-

#    This file is part of eXistDA.
#
#    Copyright (C) 2004  Sebastien PILLOZ - Ecole Normale Superieure de Lyon
#
#    eXistDA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__doc__ = """eXistDACacheManager product module."""
__version__ = "0.1-dev"

from Globals import HTMLFile, DTMLFile
from Globals import MessageDialog
from Globals import Persistent
from AccessControl import ClassSecurityInfo
from zLOG import LOG, DEBUG, INFO

import OFS.SimpleItem, OFS.ObjectManager
import Acquisition, App
import AccessControl.Role
import xmlrpclib, types, os.path
import eXistDA_utils as utils
from eXistDA_utils import iseXistURI, isHTTPURI
import eXistDA
import eXistDAresult

## Zope methods for management
manage_addeXistDACacheManagerForm = HTMLFile('dtml/eXistDACacheManagerAdd', globals())
def manage_addeXistDACacheManager(self, id, title='', storage=utils.DEFAULT_CACHEMANAGER_TYPE, path=None, maxSize = utils.DEFAULT_CACHE_MAXSIZE, REQUEST=None):
    """Add an eXistDA Cache Manager object."""
    self._setObject(id, eXistDACacheManager(id, title, storage, path, maxSize))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class eXistDACacheManager (OFS.ObjectManager.ObjectManager, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager, App.Management.Navigation):
    """An eXist CacheManager object."""
    
    meta_type = utils.EXISTDACACHEMANAGER_METATYPE
    
    isAnObjectManager=1
    
    manage_options =  OFS.ObjectManager.ObjectManager.manage_options + (
      {'label':'Properties', 'action':'manage_properties'},
      {'label':'Security', 'action':'manage_access'},
    )

    __ac_permissions__ = (
      ('View management screens', ('manage_tabs', 'manage_main', 'manage_edit')),
      ('Change permissions', ('manage_access')),
    )
    
    security = ClassSecurityInfo()
    security.declareObjectProtected("View")

    ###########
        
    def __init__(self, id, title, storage=utils.DEFAULT_CACHEMANAGER_TYPE, path=None, maxSize=DEFAULT_CACHE_MAXSIZE):
        """Initialization of the eXistDACacheManager object."""
        self.id = str(id)
        self.title = str(title)
        if storage in utils.ALL_CACHE_TYPES:
            self.storage = str(storage)
        else:
            raise utils.EXC_NOT_CACHE_TYPE
        # if we got a cache on the FS, store path to use
        if self.storage == utils.FSCACHE:
            if os.path.exists(str(path)):
                self.path = path
            else:
                raise utils.EXC_NOT_DIR
        else:
            self.path = None
        self.maxSize = int(maxSize)
        # TO DO : add cache object
        
    
    index_html = HTMLFile('dtml/index_cacheManager', globals())
    manage_main = OFS.ObjectManager.ObjectManager.manage_main
    manage_workspace = OFS.ObjectManager.ObjectManager.manage_workspace
    manage_properties = HTMLFile('dtml/eXistDACacheManagerEdit', globals())

   #Nothing can be added in the cache manager, otherwise than programmaticaly
    meta_types = []
    all_meta_types = meta_types

    ##########################################################################################
    ### Internal methods
    
    
    ### Visible methods
    security.declareProtected( utils.PERM_READ_ACCESS, 'getDoc' )
    def getDoc(self, da, xml, xsl=None, **kws):
        """Gets a document in the cache, or in the DB if it's not in cache.
        
* 'da' : the DA which is calling the CacheManager

* 'xml' : the XML document. It can be an HTTP URI, an eXist URI, a file-like object or a string

* 'xsl' : the XSL document. It can be an HTTP URI, an eXist URI, a file-like object or a string

* 'kws' : 0, 1 or more parameters that modify the output document. The names aren't important."""

        # Do MD5 computation
        # Load doc from self.cache or from eXist

    
    ##########################################################################################
    
    security.declareProtected( utils.PERM_MANAGE_ACCESS, 'manage_edit' )
    def manage_edit(self, title="", storage=utils.DEFAULT_CACHEMANAGER_TYPE, path=None, maxSize = utils.DEFAULT_CACHE_MAXSIZE, REQUEST=None):
        """Change values of parameters"""
       
        self.title = str(title)
        if storage in utils.ALL_CACHE_TYPES :
            self.storage = storage
        else:
            raise EXC_NOT_CACHE_TYPE
        if self.storage == utils.FSCACHE:
            if os.path.exists(str(path)):
                self.path = path
            else:
                raise utils.EXC_NOT_DIR
        else:
            self.path = None
        self.maxSize = int(maxSize)
    
        if REQUEST is not None:
            return MessageDialog(
                title = 'Edited',
                message = "Properties for %s changed." % self.id,
                action = './manage_main',
                )


import Globals            
Globals.InitializeClass(eXistDACacheManager)
